"use strict";
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return", awaitReturn), i[Symbol.asyncIterator] = function () { return this; }, i;
    function awaitReturn(f) { return function (v) { return Promise.resolve(v).then(f, reject); }; }
    function verb(n, f) { if (g[n]) { i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; if (f) i[n] = f(i[n]); } }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Poly = void 0;
const builder_js_1 = __importDefault(require("./builder.js"));
const poly_iterable_js_1 = __importDefault(require("./async/poly-iterable.js"));
const main_js_1 = require("./sync/main.js");
const utils_js_1 = require("./utils.js");
function* rangeUp(from, to, step) {
    for (let n = from; n < to; n += step) {
        yield n;
    }
}
function* rangeDown(from, to, step) {
    for (let n = from; n > to; n -= step) {
        yield n;
    }
}
/**
 * Main namespace for the creation of {@link PolySyncIterable} and {@link PolyAsyncIterable} objects.
 *
 * @public
 */
var Poly;
(function (Poly) {
    /**
     * Create a new {@link PolySyncIterable} object from an iterable or a function that returns iterables (such as a
     * generator function).
     *
     * If the passed argument is a function, it will be called with no arguments and its return value will be used to
     * create the resulting {@link PolySyncIterable}.
     *
     * @typeParam T - The type of the resulting iterable
     * @param iterableOrFactory - an iterable or iterable factory used to create the resulting iterable
     * @returns A {@link PolySyncIterable} that iterates over the same elements as the passed iterable
     *
     * @public
     */
    function syncFrom(iterableOrFactory) {
        if (iterableOrFactory instanceof main_js_1.PolySyncIterable) {
            return iterableOrFactory;
        }
        const iterable = (typeof iterableOrFactory === 'function') ? iterableOrFactory() : iterableOrFactory;
        if ((0, utils_js_1.isSyncIterable)(iterable)) {
            return new main_js_1.IterablePolySyncIterable(iterable);
        }
        throw Error('argument is not iterable');
    }
    Poly.syncFrom = syncFrom;
    /**
     * Create a new {@link PolyAsyncIterable} object from an iterable, async iterable, or a function that returns
     * iterables (such as a generator function or an async generator function).
     *
     * If the passed argument is a function, it will be called with no arguments and its return value will be used to
     * create the resulting {@link PolyAsyncIterable}.
     *
     * @typeParam T - The type of the resulting async iterable
     * @param iterableOrFactory - an async iterable or async iterable factory used to create the resulting async iterable
     * @returns A {@link PolyAsyncIterable} that iterates over the same elements as the passed iterable
     *
     * @public
     */
    function asyncFrom(iterableOrFactory) {
        if (iterableOrFactory instanceof poly_iterable_js_1.default) {
            return iterableOrFactory;
        }
        const iterable = (typeof iterableOrFactory === 'function') ? iterableOrFactory() : iterableOrFactory;
        if ((0, utils_js_1.isAsyncIterable)(iterable)) {
            return new poly_iterable_js_1.default(iterable);
        }
        if ((0, utils_js_1.isSyncIterable)(iterable)) {
            return new main_js_1.IterablePolySyncIterable(iterable).async();
        }
        throw Error('argument is not sync or async iterable');
    }
    Poly.asyncFrom = asyncFrom;
    function from(iterableOrFactory) {
        if (iterableOrFactory instanceof main_js_1.PolySyncIterable || iterableOrFactory instanceof poly_iterable_js_1.default) {
            return iterableOrFactory;
        }
        const iterable = (typeof iterableOrFactory === 'function') ? iterableOrFactory() : iterableOrFactory;
        if ((0, utils_js_1.isAsyncIterable)(iterable)) {
            return new poly_iterable_js_1.default(iterable);
        }
        if ((0, utils_js_1.isSyncIterable)(iterable)) {
            return new main_js_1.IterablePolySyncIterable(iterable);
        }
        throw Error('argument is not sync or async iterable');
    }
    Poly.from = from;
    /**
     * Return a new {@link AsyncIterableBuilder}, an iterable object that can be constructed by calling its methods.
     *
     * @remarks
     * This method for creating iterables should only be used as last resort if other functions are not enough.
     * In particular, before using this method, see if {@link Poly.buildWith} works for you.
     *
     * @typeParam T - The type of the resulting builder
     *
     * @public
     */
    function builder() {
        return new builder_js_1.default();
    }
    Poly.builder = builder;
    /**
     * Create a new {@link PolyAsyncIterable} by passing the `func` function an {@link AsyncIterableBuilder} object.
     *
     * @remarks
     * This method for creating iterables should only be used as a last resort if other functions are not enough.
     *
     * @typeParam T - The type of the resulting async iterable
     * @param func - A function that will receive an object containing the methods `value`, `error` and `done`.
     *
     * @public
     */
    function buildWith(func) {
        const bld = builder();
        func(bld);
        return Poly.asyncFrom(bld);
    }
    Poly.buildWith = buildWith;
    /**
     * Return a {@link PolySyncIterable} that yields no elements upon iteration.
     *
     * @typeParam T - Type of the resulting iterable
     * @returns An iterable that yields no elements
     *
     * @public
     */
    function empty() {
        return syncFrom([]);
    }
    Poly.empty = empty;
    /**
     * Returns a {@link PolySyncIterable} that yields the same elements as `Object.entries` would.
     *
     * @typeParam T - Type of the resulting iterable
     * @returns An iterable that yields the entries of the passed object
     *
     * @public
     */
    function entries(obj) {
        return syncFrom(function* () {
            for (const key in obj) {
                if (Object.prototype.hasOwnProperty.call(obj, key)) {
                    yield [key, obj[key]];
                }
            }
        });
    }
    Poly.entries = entries;
    /**
     * Returns a {@link PolySyncIterable} that yields the same elements as `Object.keys` would.
     *
     * @typeParam T - Type of the resulting iterable
     * @returns An iterable that yields the keys of the passed object
     *
     * @public
     */
    function keys(obj) {
        return syncFrom(function* () {
            for (const key in obj) {
                if (Object.prototype.hasOwnProperty.call(obj, key)) {
                    yield key;
                }
            }
        });
    }
    Poly.keys = keys;
    /**
     * Returns a {@link PolySyncIterable} that yields the same elements as `Object.values` would.
     *
     * @typeParam T - Type of the resulting iterable
     * @returns An iterable that yields the values of the passed object
     *
     * @public
     */
    function values(obj) {
        return syncFrom(function* () {
            for (const key in obj) {
                if (Object.prototype.hasOwnProperty.call(obj, key)) {
                    yield obj[key];
                }
            }
        });
    }
    Poly.values = values;
    function range(fromOrTo, maybeTo, step = 1) {
        const fromNum = (maybeTo === undefined) ? 0 : fromOrTo;
        const toNum = (maybeTo === undefined) ? fromOrTo : maybeTo;
        if (step === 0) {
            throw new Error('"step" can\'t be 0');
        }
        const rangeFunc = (step > 0 ? rangeUp : rangeDown);
        return syncFrom(rangeFunc(fromNum, toNum, Math.abs(step)));
    }
    Poly.range = range;
    /**
     * Returns a {@link PolySyncIterable} that yields the passed argument forever, without ending.
     *
     * @typeParam T - The type of the interable
     * @param value - The reapeated element
     * @returns an infinite iterable over the same element
     *
     * @public
     */
    function repeat(value) {
        return syncFrom(function* () {
            while (true) {
                yield value;
            }
        });
    }
    Poly.repeat = repeat;
    function syncIterate(func, initValue) {
        return syncFrom(function* () {
            let nextValue = initValue;
            while (true) {
                nextValue = func(nextValue);
                yield nextValue;
            }
        });
    }
    Poly.syncIterate = syncIterate;
    function asyncIterate(func, initValue) {
        return asyncFrom(function () {
            return __asyncGenerator(this, arguments, function* () {
                let nextValue = initValue;
                while (true) {
                    nextValue = yield __await(func(nextValue));
                    yield yield __await(nextValue);
                }
            });
        });
    }
    Poly.asyncIterate = asyncIterate;
})(Poly || (exports.Poly = Poly = {}));
//# sourceMappingURL=poly.js.map