"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sortGen = exports.reverseGen = exports.uniqueGen = exports.groupByGen = exports.chunkWhileGen = exports.chunkGen = exports.flattenGen = exports.tapGen = exports.mapGen = exports.filterGen = exports.takeWhileGen = exports.dropWhileGen = exports.sliceGen = exports.takeLastGen = exports.dropLastGen = exports.takeGen = exports.dropGen = exports.prependGen = exports.appendGen = void 0;
function* appendGen(iterA, iterB) {
    yield* iterA;
    yield* iterB;
}
exports.appendGen = appendGen;
function* prependGen(iterA, iterB) {
    yield* iterB;
    yield* iterA;
}
exports.prependGen = prependGen;
function* dropGen(iter, num) {
    let rem = num;
    for (const elem of iter) {
        if (rem-- <= 0) {
            yield elem;
        }
    }
}
exports.dropGen = dropGen;
function* takeGen(iter, num) {
    let rem = num;
    for (const elem of iter) {
        if (rem-- <= 0) {
            return;
        }
        yield elem;
    }
}
exports.takeGen = takeGen;
function* dropLastGen(iter, num) {
    const buf = Array(num);
    let pos = 0;
    let size = 0;
    for (const elem of iter) {
        if (size >= num) {
            yield num ? buf[pos] : elem;
        }
        else {
            size++;
        }
        if (num) {
            buf[pos] = elem;
            pos = ((pos + 1) % num);
        }
    }
}
exports.dropLastGen = dropLastGen;
function* takeLastGen(iter, num) {
    const buf = Array(num);
    let pos = 0;
    let size = 0;
    for (const elem of iter) {
        size = size >= num ? num : size + 1;
        if (num) {
            buf[pos] = elem;
            pos = ((pos + 1) % num);
        }
    }
    pos = size >= num ? pos : 0;
    for (let i = 0; i < size; i++) {
        yield buf[pos];
        pos = ((pos + 1) % num);
    }
}
exports.takeLastGen = takeLastGen;
function* sliceNegPosGen(iter, start, end) {
    const buf = Array(start);
    let pos = 0;
    let size = 0;
    let num = end;
    for (const elem of iter) {
        if (size >= start && end) {
            num--;
        }
        size = size >= start ? start : size + 1;
        if (start) {
            buf[pos] = elem;
            pos = ((pos + 1) % start);
        }
    }
    pos = size >= start ? pos : 0;
    for (let i = 0; i < Math.min(size, num); i++) {
        yield buf[pos];
        pos = ((pos + 1) % start);
    }
}
function sliceGen(iter, start, end) {
    if (start >= 0 && end == null) {
        return dropGen(iter, start);
    }
    else if (end == null) { // && start < 0
        return takeLastGen(iter, -start);
    }
    else if (start >= 0 && end >= 0) {
        return takeGen(dropGen(iter, start), end - start);
    }
    else if (start < 0 && end >= 0) {
        return sliceNegPosGen(iter, -start, end);
    }
    else if (start >= 0) { // && end < 0
        return dropLastGen(dropGen(iter, start), -end);
    }
    else { // start < 0 && end < 0
        return dropLastGen(takeLastGen(iter, -start), -end);
    }
}
exports.sliceGen = sliceGen;
function* dropWhileGen(iter, func) {
    let idx = 0;
    let yielding = false;
    for (const elem of iter) {
        if (yielding || !func(elem, idx++)) {
            yielding = true;
            yield elem;
        }
    }
}
exports.dropWhileGen = dropWhileGen;
function* takeWhileGen(iter, func) {
    let idx = 0;
    for (const elem of iter) {
        if (!func(elem, idx++)) {
            return;
        }
        yield elem;
    }
}
exports.takeWhileGen = takeWhileGen;
function* filterGen(iter, func) {
    let idx = 0;
    for (const elem of iter) {
        if (func(elem, idx++)) {
            yield elem;
        }
    }
}
exports.filterGen = filterGen;
function* mapGen(iter, func) {
    let idx = 0;
    for (const elem of iter) {
        yield func(elem, idx++);
    }
}
exports.mapGen = mapGen;
function* tapGen(iter, func) {
    let idx = 0;
    for (const elem of iter) {
        func(elem, idx++);
        yield elem;
    }
}
exports.tapGen = tapGen;
function* flattenGen(iter) {
    for (const elem of iter) {
        yield* elem;
    }
}
exports.flattenGen = flattenGen;
function* chunkGen(iter, num) {
    let chunk = [];
    for (const elem of iter) {
        chunk.push(elem);
        if (chunk.length === num) {
            yield chunk;
            chunk = [];
        }
    }
    if (chunk.length) {
        yield chunk;
    }
}
exports.chunkGen = chunkGen;
function* chunkWhileGen(iter, func) {
    let chunk = [];
    for (const elem of iter) {
        if (chunk.length === 0 || func(elem, chunk[chunk.length - 1], chunk[0])) {
            chunk.push(elem);
        }
        else {
            yield chunk;
            chunk = [elem];
        }
    }
    if (chunk.length) {
        yield chunk;
    }
}
exports.chunkWhileGen = chunkWhileGen;
function* groupByGen(iter, func) {
    var _a;
    const groups = new Map();
    let idx = 0;
    for (const elem of iter) {
        const key = func(elem, idx++);
        const group = (_a = groups.get(key)) !== null && _a !== void 0 ? _a : [];
        group.push(elem);
        groups.set(key, group);
    }
    yield* groups.entries();
}
exports.groupByGen = groupByGen;
function* uniqueGen(iter, func) {
    const seen = new Set();
    let idx = 0;
    for (const elem of iter) {
        const key = func(elem, idx++);
        if (!seen.has(key)) {
            yield elem;
            seen.add(key);
        }
    }
}
exports.uniqueGen = uniqueGen;
function* reverseGen(iter) {
    const buf = Array.from(iter);
    for (let i = buf.length - 1; i >= 0; i--) {
        yield buf[i];
    }
}
exports.reverseGen = reverseGen;
function* sortGen(iter, func) {
    const buf = Array.from(iter);
    for (const elem of buf.sort(func)) {
        yield elem;
    }
}
exports.sortGen = sortGen;
//# sourceMappingURL=generators.js.map