import Benchmark from 'benchmark';
import Poly from '../lib/main.js';
const NAME_ARRAY = 'Array#map';
const NAME_POLY = 'PolySyncIterable#map';
const mapOnce = (mappable) => mappable.map((n) => n ** 2);
const mapTwice = (mappable) => mappable.map((n) => n ** 2).map((n) => n % 17);
const mapMany = (mappable) => mappable
    .map((n) => n ** 2)
    .map((n) => n % 17)
    .map((n) => n + 10)
    .map((n) => n / 2)
    .map((n) => Math.sqrt(n));
function arrayOf(n) {
    return Array(n).fill(null).map((v, i) => i);
}
function benchFor(num, runMap) {
    const array = arrayOf(num);
    const suite = new Benchmark.Suite(`${num} elements - ${runMap.name}`);
    suite.add(NAME_ARRAY, () => runMap(array));
    suite.add(NAME_POLY, () => runMap(Poly.syncFrom(array)).toArray());
    suite.on('cycle', (event) => {
        console.log('·', String(event.target));
    });
    suite.on('complete', () => {
        const array = suite.filter((elem) => elem.name === NAME_ARRAY)[0];
        const poly = suite.filter((elem) => elem.name === NAME_POLY)[0];
        const ratio = poly.hz / array.hz;
        console.log('Ratio: %s%', (ratio * 100).toFixed(2));
    });
    console.log(suite.name);
    suite.run({});
    console.log();
}
const SIZES = [10, 1000, 100000];
for (const size of SIZES) {
    benchFor(size, mapOnce);
    benchFor(size, mapTwice);
    benchFor(size, mapMany);
}
//# sourceMappingURL=map.js.map