import PolySyncIterable from './poly-sync-iterable.js';
class MapSyncIterator {
    #mapper;
    #iterator;
    #index = 0;
    constructor(iterator, mapper) {
        this.#iterator = iterator;
        this.#mapper = mapper;
    }
    next() {
        const elem = this.#iterator.next();
        return elem?.done ? elem : { value: this.#mapper(elem.value, this.#index++) };
    }
}
function combinedIndexedMapping(funcA, funcB) {
    return (orig, idx) => funcB(funcA(orig, idx), idx);
}
export default class MapSyncIterable extends PolySyncIterable {
    #iterable;
    #mapper;
    constructor(iterable, mapper) {
        super();
        this.#iterable = iterable;
        this.#mapper = mapper;
    }
    static create(iterable, mapper) {
        return new MapSyncIterable(iterable, mapper);
    }
    [Symbol.iterator]() {
        return new MapSyncIterator(this.#iterable[Symbol.iterator](), this.#mapper);
    }
    map(func) {
        return new MapSyncIterable(this.#iterable, combinedIndexedMapping(this.#mapper, func));
    }
}
//# sourceMappingURL=map-sync-iterable.js.map