export function identity(x) {
    return x;
}
export async function asyncIdentity(x) {
    return x;
}
export function comparator(a, b) {
    return (a < b) ? -1 : (a > b) ? +1 : 0;
}
export function isNotNullish(obj) {
    return obj != null;
}
export function isSyncIterable(obj) {
    return typeof obj == 'object' && obj != null && Symbol.iterator in obj;
}
export function isAsyncIterable(obj) {
    return typeof obj == 'object' && obj != null && Symbol.asyncIterator in obj;
}
export const asserts = {
    isFunction(arg, name = 'argument') {
        if (typeof arg !== 'function') {
            throw new Error(`${name} should be a function`);
        }
        return true;
    },
    isInteger(arg, name = 'argument') {
        if (!Number.isInteger(arg)) {
            throw new Error(`${name} should be an integer`);
        }
        return true;
    },
    isPositiveInteger(arg, name = 'argument') {
        if (!Number.isInteger(arg) || arg <= 0) {
            throw new Error(`${name} should be a positive integer`);
        }
        return true;
    },
    isNonNegativeInteger(arg, name = 'argument') {
        if (!Number.isInteger(arg) || arg < 0) {
            throw new Error(`${name} should be a non-negative integer`);
        }
        return true;
    },
    isSyncIterable(arg, name = 'argument') {
        const iteratorMethod = arg[Symbol.iterator];
        if (typeof iteratorMethod !== 'function') {
            throw new Error(`${name} should be sync iterable`);
        }
        return true;
    },
    isAsyncIterable(arg, name = 'argument') {
        const asyncIteratorMethod = arg[Symbol.asyncIterator];
        if (typeof asyncIteratorMethod !== 'function') {
            throw new Error(`${name} should be async iterable`);
        }
        return true;
    },
    isSyncOrAsyncIterable(arg, name = 'argument') {
        const iteratorMethod = arg[Symbol.iterator];
        const asyncIteratorMethod = arg[Symbol.asyncIterator];
        if (typeof iteratorMethod !== 'function' && typeof asyncIteratorMethod !== 'function') {
            throw new Error(`${name} should be sync or async iterable`);
        }
        return true;
    },
};
//# sourceMappingURL=utils.js.map